//#define RT_NOAUDIO

using UnityEngine;
using System.Collections;
using System;
using System.IO;
using System.Text;
using UnityEngine.UI;
using System.Collections.Generic;


public class GameLogic : MonoBehaviour 
{
	static GameLogic m_this = null;
	public WorldGenerator m_worldGen = null;
	public GameObject m_player;
	public Camera m_camera = null;
	public PlayerData m_playerData = null;
	public Player m_playerScript = null;
	public Text m_logText = null;
	int C_MAX_LINES_IN_LOG = 10;
	const float cameraZoomNormal = 5;
	const float cameraZoomOld = 5; //set to 8 for weird

	public LinkedList<string> m_textList = new LinkedList<string>();

	public enum eMode
	{
		STOPPED,
		STARTED,
		DEAD

	}
	public eMode m_mode = eMode.STOPPED;

	public eMode GetMode() { return m_mode; }
	public static string GetName()
	{
		return Get ().name;
	}

	public static void Log(string text)
	{
		Get().AddLog(text);
	}

	
	void AddLog(string text)
	{
		if (m_textList.Count > 0)
		{
			//Debug.Log("Comparing " + m_textList.Last.Value + " and " + text);

			if (m_textList.Last.Value.CompareTo("\n"+text) == 0)
			{
				return; //skip it, it's a dupe text
			}
		}
		m_textList.AddLast("\n" + text);
		
		if (m_textList.Count > C_MAX_LINES_IN_LOG)
		{
			m_textList.RemoveFirst();
		}
		m_logText.text = "";

		string temp = "";

		foreach(string s in m_textList)
		{
			temp += s;
		}
		m_logText.text = temp;
	}

	public Camera GetCamera()
	{
		return m_camera;
	}
	public GameObject GetPlayer()
	{
		return m_player;
	}
	public Player GetPlayerScript()
	{
		return m_playerScript;
	}

	static public GameLogic Get()
	{
		return m_this;
	}
	public PlayerData GetPlayerData()
	{
		return m_playerData;
	}

	// Use this for initialization
	void Start () 
	{
		m_this = this;
		m_logText.text = "";
		//DebugConsole.instance.maxMessages = 6;

		//DebugConsole.Log("App starting");
#if RT_NOAUDIO
		AudioListener.pause = true;
#endif

		//Debug.Log("Generating world...");
		GameLogic.Log("Rogue-Life by Seth A. Robinson (code) and Akiko Robinson (art) for LD 34. Sfx ripped from Growtopia.");

		m_worldGen.GenerateWorld();
		//RTEventManager.Get().Schedule(RTAudioManager.GetName(), "PlayMusic", 1, "intro");
	}
     
	void OnApplicationQuit() 
	{
		// Make sure prefs are saved before quitting.
		PlayerPrefs.Save();
	}

	void OnGUI() 
	{
	}

	public void StartGame()
	{
		GetCamera().orthographicSize = cameraZoomNormal;
		m_mode = GameLogic.eMode.STARTED;
		RTAudioManager.Get().Play("music_box");
		Log("You are born. Use WASD or arrow keys to push against life choices. Find all five 'Meanings'. If you give up, press Q. Use the mouse wheel to zoom in/out.");
	}

	public void OldManView()
	{

		GetCamera().orthographicSize = cameraZoomOld;
	}

	// Update is called once per frame
	void Update () 
    {
	}

	public void OnDie(string text)
	{
		m_mode = eMode.DEAD;

		GetPlayerScript().OnDie();
		RTAudioManager.Get().Play("punch_organic");
		Log(text);

		int totalMeanings = GameLogic.Get().GetPlayerScript().m_totalMeanings;

			string finishText = "GAME OVER! You found " + GameLogic.Get().GetPlayerScript().m_meaningsFound + " of " +
				totalMeanings + " meanings. Was it a life well lived?  That's up to you to decide. Click reload in your browser for another go.";

		Log(finishText);

		}

	}
