using UnityEngine;
using System.Collections;
using UnityEngine.UI;


public class Player : MonoBehaviour
{

	public int m_totalMeanings = 5;
	public int m_meaningsFound = 0;

	public SpriteRenderer m_spriteRenderer; //what the player looks like
	public Animator m_animator;

	public AnimatorOverrideController[] m_animOverrides;

	// Use this for initialization
	void Start ()
	{
		m_meaningsFound = 0;

		m_animator.runtimeAnimatorController = m_animOverrides[0];
	}
	
	void UpdatePlayerGraphic()
	{
		if (GameLogic.Get().GetMode() == GameLogic.eMode.DEAD)
		{

			return;
		}

		RuntimeAnimatorController oldController = m_animator.runtimeAnimatorController;

        float age = GameLogic.Get().GetPlayerData().GetAge();
		RuntimeAnimatorController controller = null;

		string msg = "Error";
		float strengthMod = 0;
		float selfMod = 0;
		float excitementMod = 0;

		
		if (age > 55)
		{
			controller = m_animOverrides[4];
			msg = "Old age has caused your body to whither and your eyesight to falter, yet you have a greater perspective on life.";
			strengthMod = -40;
			//GameLogic.Get().OldManView();

		} else if (age > 28)
		{
			controller = m_animOverrides[3];
			msg = "Working long hours as an adult has lowered your excitement.";
			excitementMod = -20;
		}
		else if (age > 14)
		{
			controller = m_animOverrides[2];
			msg = "Uncertainty during your teenage years has lowered your Self Acceptance, but gained you Physical.";
			selfMod = -15;
			strengthMod = 10;
		}
		else if (age > 6)
		{
			controller = m_animOverrides[1];
			msg = "You are now a toddler! Your Physical has increased.";
			strengthMod = 10;

		} else
		{
			controller = m_animOverrides[0];
		}

		if (oldController != controller)
		{
			//age change
			RTAudioManager.Get().Play("growing");
			GameLogic.Log(msg);
			GameLogic.Get().GetPlayerData().ModSelfSafe(selfMod);
			GameLogic.Get().GetPlayerData().ModStrengthSafe(strengthMod);
			GameLogic.Get().GetPlayerData().ModExcitementSafe(excitementMod);
			m_animator.runtimeAnimatorController = controller;
		}

	}

	// Update is called once per frame
	void Update ()
	{
		UpdatePlayerGraphic();

		if (GameLogic.Get().GetMode() != GameLogic.eMode.STARTED) return;

		float mouseY = -(int)Input.mouseScrollDelta.y;

		//Debug.Log("Y is " + mouseY);

		float zoomSpeed = 0.1f;
		while (mouseY != 0)
		{
			float zoomMod = 1 + zoomSpeed;

			if (mouseY > 0)
			{
				mouseY -= 1;
			}
			else
			{
				mouseY += 1;
				zoomMod = 1 + -zoomSpeed;
			}

			//apply it
			float size = GameLogic.Get().GetCamera().orthographicSize;
			size *= zoomMod;
			if (size < 1) size = 1;
			GameLogic.Get().GetCamera().orthographicSize = size;

		}

	}

	void FixedUpdate()
	{
		if (GameLogic.Get().GetMode() != GameLogic.eMode.STARTED) return;
		
		if (Input.GetKeyDown(KeyCode.Q))
		{
			GameLogic.Get().OnDie("Suicide - You've decided to end your life. You don't understand how your choices have lead you to such self loathing and despair, but yet, here you are.");
			return;
		}


		//m_animator.runtimeAnimatorController = m_animOverrides[2];

		bool bMoving = false;

		const float joystickPlay = 0.3f;
		if (Input.GetAxis("Horizontal") < -joystickPlay)
		{
		//	Debug.Log("LEFT!" + Input.GetAxis("Horizontal"));
			m_spriteRenderer.flipX = false;
			bMoving = true;
        }
		if (Input.GetAxis("Horizontal") > joystickPlay)
		{
		//	Debug.Log("RIGHT!" + Input.GetAxis("Horizontal"));
			m_spriteRenderer.flipX = true;
			bMoving = true;
		}
		if (Input.GetAxis("Vertical") > joystickPlay)
		{
		
			bMoving = true;
		}

		if (Input.GetAxis("Vertical") < -joystickPlay)
		{

			bMoving = true;
		}

	
		if (bMoving)
		{
			m_animator.Play("Walk", -1);
		}
		else
		{
			m_animator.Play("Stop", -1, 0);
		}

		Vector3 pos = gameObject.transform.position;

		float speed = 0.1f;

		pos.x += Input.GetAxis("Horizontal") * speed;
		pos.y += Input.GetAxis("Vertical") * speed;
		transform.position = pos;


		//GameLogic.Get().OldManView();
	}

	public void OnDie()
	{
		m_animator.Play("Die", -1, 0);
		m_spriteRenderer.flipX = false;
	}
}
