using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class PlayerData : MonoBehaviour
{

	float m_age = 1;
	float m_maxAge = 88;
	public Text m_ageText;
	public Image m_ageBarImage;

	float m_wealth = 0;
	float m_maxWealth = 10000;
	public Text m_wealthText;
	public Image m_wealthBarImage;

	float m_skill = 0;
	float m_maxSkill = 100;
	public Text m_skillText;
	public Image m_skillBarImage;

	float m_strength = 0;
	float m_maxStrength = 100;
	public Text m_strengthText;
	public Image m_strengthBarImage;

	float m_self = 0;
	float m_maxSelf = 100;
	public Text m_selfText;
    public Image m_selfBarImage;

	float m_excitement = 0;
	float m_maxExcitement = 100;
	public Text m_excitementText;
	public Image m_excitementBarImage;

	const float m_idleAgeIncrease = 0.005f; //0 for no age increase


	// Use this for initialization
	void Start ()
	{
		SetWealth(0);
		SetStrength(20);
		SetSelf(20);
		SetSkill(1);
		SetExcitement(0);

	}

	void UpdateWealthDisplay()
	{
		m_wealthText.text ="Wealth: $" + m_wealth;
		UpdateBarDisplay(m_wealthBarImage, m_wealth, m_maxWealth);
	}

	public void SetWealth(float wealth)
	{
		m_wealth = wealth;
		UpdateWealthDisplay();
	}

	public void ModWealth(float wealth)
	{
		m_wealth += wealth;
		UpdateWealthDisplay();
	}

	public float GetWealth()
	{
		return m_wealth;
	}

	public void ModSkill(float skill)
	{
		m_skill += skill;
		UpdateSkillDisplay();
	}

	void UpdateSkillDisplay()
	{
		m_skillText.text = "Skill: " + RTUtil.RoundToOneDec(m_skill);
		UpdateBarDisplay(m_skillBarImage, m_skill, m_maxSkill);
	}

	public void SetSkill(float skill)
	{
		m_skill = skill;
		UpdateSkillDisplay();
	}

	public float  GetSkill()
	{
		return m_skill;
	}

	public void SetWhite(Text t)
	{
		t.color = Color.white;
	}

	public void FailSkill()
	{
		Fail(m_skillText);
	}
	public void FailSelf()
	{
		Fail(m_selfText);
	}
	public void FailWealth()
	{
		Fail(m_wealthText);
	}
	public void FailStrength()
	{
		Fail(m_strengthText);
	}
	public void FailExcitement()
	{
		Fail(m_excitementText);
	}

	public void Fail(Text t)
	{
		t.color = Color.red;
		RTEventManager.Get().Schedule("MainPlayer", "SetWhite", 0.1f, t);
	}
	public void ModExcitement(float excitement)
	{
		m_excitement += excitement;
		UpdateExcitementDisplay();
	}

	void UpdateExcitementDisplay()
	{
		m_excitementText.text = "Excitement: " + RTUtil.RoundToOneDec(m_excitement);
		UpdateBarDisplay(m_excitementBarImage, m_excitement, m_maxExcitement);
	}

	public void SetExcitement(float excitement)
	{
		m_excitement = excitement;
		UpdateExcitementDisplay();
	}

	public float GetExcitement()
	{
		return m_excitement;
	}

	public void ModStrength(float strength)
	{
		m_strength += strength;
		UpdateStrengthDisplay();
	}

	public void ModStrengthSafe(float strength)
	{
		m_strength += strength;
		if (m_strength < 1) m_strength = 1;
		UpdateStrengthDisplay();
	}

	public void ModExcitementSafe(float excitement)
	{
		m_excitement += excitement;
		if (m_excitement < 0) m_excitement = 0;
		UpdateExcitementDisplay();
	}

	void UpdateStrengthDisplay()
	{
		m_strengthText.text = "Physical: " + RTUtil.RoundToOneDec(m_strength);
		UpdateBarDisplay(m_strengthBarImage, m_strength, m_maxStrength);
	}

	public float GetStrength()
	{
		return m_strength;
	}

	public void SetStrength(float strength)
	{
		m_strength = strength;
		UpdateStrengthDisplay();
	}


	void UpdateSelfDisplay()
	{
		m_selfText.text = "Self Acceptance: " + RTUtil.RoundToOneDec(m_self);
		UpdateBarDisplay(m_selfBarImage, m_self, m_maxSelf);
	}

	public void SetSelf(float self)
	{
		m_self = self;
		UpdateSelfDisplay();
	}

	public float GetSelf()
	{
		return m_self;
	}

	public void ModSelf(float self)
	{
		m_self += self;
		UpdateSelfDisplay();
	}


	public void ModSelfSafe(float self)
	{
		m_self += self;
		if (m_self < 1) m_self = 1;
		UpdateSelfDisplay();
	}

	public void ModAge(float ageMod)
	{
		m_age += ageMod;
	}
	
	// Update is called once per frame
	void Update ()
	{
	
	}

	public float GetAge() { return m_age; }

	void UpdateBarDisplay(Image im, float cur, float max)
	{
		im.fillAmount = cur / max;
	}

	
	void FixedUpdate()
	{
		if (GameLogic.Get().m_mode == GameLogic.eMode.STARTED)
		{
			m_age += m_idleAgeIncrease;
		
			m_ageText.text = "Age: " + RTUtil.RoundToOneDec(m_age);
			UpdateBarDisplay(m_ageBarImage, m_age, m_maxAge);

			if (m_age >= m_maxAge)
			{
				GameLogic.Get().OnDie("You've died of old age.");
			}

			if (m_strength < 1)
			{
				GameLogic.Get().OnDie("Sadly your body has failed you. You've died in a hospital due to your frail health.");
			}

			/*
			if (m_self <= 0)
			{
				GameLogic.Get().OnDie("You've decided to end your life. You don't understand how your choices have lead you to such self loathing and despair, but yet, here you are.");
			}
			*/
		}


	}
}
