using UnityEngine;
using System.Collections;
using System.Collections.Generic;

/*
	
Created by Seth A. Robinson - rtsoft.com 2013
	
 TO SETUP:
 
 Create a GameObject in unity called RTAudioManager and attach this script 
 
To call from any script:

//to play a sfx.. must be less then 8 seconds for no good reason
//Note: Don't add the .wav part of the filename

RTAudioManager.Get().Play("sfxFilename");

//to schedule it using RTEventManager to play in X seconds:
			
RTEventManager.Get().Schedule(RTAudioManager.GetName(), "Play", 1, "sfxFilename");

//to play a mp3 song (song or its folders must be placed off of Assets/Resources)
//this will replace any existing music
RTAudioManager.Get().PlayMusic("audio/wiz", 1.0f, 1.0f, true);

//To use with RTEventManager to play the same song 2 seconds later   
RTEventManager.Get().Schedule(RTAudioManager.GetName(), "PlayMusic", 2, "musicFileName");

//Play the song 2 seconds later, but with settings for vol, pitch, and looping
RTEventManager.Get().Schedule(RTAudioManager.GetName(), "PlayMusicEx", 2, new RTDB("fileName", "chalk",
			"volume", 1.0f, "pitch", 2.0f,  "loop", false));

 
  //Example of sending multiple parms with RTAudioManager's PlayEx function:
RTEventManager.Get().Schedule(RTAudioManager.GetName(), "PlayEx", 2, new RTDB("fileName", "chalk",
			"volume", 1.0f, "pitch", 2.0f, "loop", true));
			
Note: If a parm is missing with PlayEx the default is used instead

Note: .wav or .mp3/ogg etc must be located in the Assets/Resources so unity includes them automatically.
They must NOT be marked as 3d sounds, if you hear nothing but don't see an error, this is what's wrong.
 
*/ 


public class RTAudioManager : MonoBehaviour 
{
	static RTAudioManager m_instance = null;
	public AudioSource m_activeMusic = null;
	

	RTAudioManager()
	{
			m_instance = this;
	}
	

	public void Start()
	{
		gameObject.name = "RTAudioManager";
		//Debug.Log("RTAudioManager initted, gameobject we're in renamed to RTAudioManager");
	}
	
	public void PlayEx(RTDB db)
	{
		PlayEx(db.GetString("fileName"),
			db.GetFloatWithDefault("volume", 1.0f),
			db.GetFloatWithDefault("pitch", 1.0f)
			);
	}

	public void PlayMusicEx(RTDB db)
	{
		PlayMusic(db.GetString("fileName"),
		       db.GetFloatWithDefault("volume", 1.0f),
		       db.GetFloatWithDefault("pitch", 1.0f),
		       db.GetBoolWithDefault("loop", true)
		       );
	}
	
	public void PlayMusic(string fileName)
	{
		PlayMusic(fileName, 1, 1, true);
	}

	public void StopMusic()
	{
		if (m_activeMusic != null)
		{
			//Debug.Log ("Stopping music");
            m_activeMusic.Stop();
			Destroy(m_activeMusic);
			m_activeMusic = null;
        }
        
	}

	public void PlayMusic(string fileName, float vol, float pitch, bool loop)
	{

		if (Camera.main != null)
		{
			AudioListener al = (AudioListener) Camera.main.GetComponent("AudioListener");
			transform.parent = al.transform;
			transform.position = Camera.main.transform.position;
		}

		//Debug.Log ("Playing music "+fileName);
		AudioClip clip = Resources.Load(fileName) as AudioClip;
 	 	
		if (clip == null)
		{
			Debug.LogWarning ("Couldn't find "+fileName);
			return;
		}

		if (m_activeMusic != null)
		{
			//Debug.Log ("Stopping music");
			m_activeMusic.Stop();
			//Destroy(m_activeMusic);
			m_activeMusic = null;
		}
		m_activeMusic = gameObject.AddComponent<AudioSource>() as AudioSource;
		GetComponent<AudioSource>().volume = vol;
		GetComponent<AudioSource>().pitch = pitch;
		GetComponent<AudioSource>().loop = loop;
		GetComponent<AudioSource>().clip = clip;
		GetComponent<AudioSource>().Play();
	}
	
	public AudioSource GetMusicComponent()
	{
		return m_activeMusic;
    }

	public void PlayEx(string fileName, float vol, float pitch)
	{
		//Debug.Log ("Playing sfx "+fileName);
		AudioClip clip = Resources.Load(fileName) as AudioClip;
 	 	
		if (clip == null)
		{
			Debug.LogWarning ("Couldn't find "+fileName);
			return;
		}
		GameObject o = new GameObject("sfx");

		if (Camera.main != null)
		{
			AudioListener al = (AudioListener) Camera.main.GetComponent("AudioListener");
			o.transform.parent = al.transform;
			o.transform.position = Camera.main.transform.position;
		}
		o.AddComponent<AudioSource>();
		o.GetComponent<AudioSource>().volume = vol;
		o.GetComponent<AudioSource>().pitch = pitch;
		o.GetComponent<AudioSource>().PlayOneShot(clip);
	
		GameObject.Destroy(o, clip.length); //kill it in a bit.. todo: this time should be the actual time the clip takes to play..
	}


	public void Play(string fileName)
	{
		PlayEx(fileName, 1, 1);
	}

	// Update is called once per frame
	void Update () 
	{
	
	}
	
	public static RTAudioManager Get()
	{
		if (!m_instance)
		{
			Debug.LogWarning("RTAudioManager: Huh?  You need to attach this script to a gameobject first.");
		}
		return m_instance;
	}
	
	public static string GetName()
	{
		return Get ().name;
	}
	
	
}
