﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;


public class RTDB 
{

	//these match Proton's variant types.. entity and component are unused here in unity though
	public const int TYPE_UNUSED=0;
	public const int TYPE_FLOAT=1;
	public const int TYPE_STRING=2;
	public const int TYPE_VECTOR2=3;
	public const int TYPE_VECTOR3=4;
	public const int TYPE_UINT32=5;
	public const int TYPE_ENTITY=6;
	public const int TYPE_COMPONENT=7;
	public const int TYPE_RECT=8;
	public const int TYPE_INT32=9;

	Dictionary<string, object> m_database = new Dictionary<string, object>();

	public RTDB()
	{
	}

	public void Clear()
	{
		m_database.Clear();
	}
	public RTDB(string key1, object value1)
	{
		m_database[key1] = value1;
	}
	
	public RTDB(string key1, object value1, string key2, object value2)
	{
		m_database[key1] = value1;
		m_database[key2] = value2;
	}

	public RTDB(string key1, object value1, string key2, object value2, string key3, object value3)
	{
		m_database[key1] = value1;
		m_database[key2] = value2;
		m_database[key3] = value3;
	}

	public RTDB(string key1, object value1, string key2, object value2, string key3, object value3, string key4, object value4)
	{
		m_database[key1] = value1;
		m_database[key2] = value2;
		m_database[key3] = value3;
		m_database[key4] = value4;
	}
	
	public object Get(string key)
	{
		return m_database[key];	
	}

	public int GetInt32(string key)
	{
		return (int)m_database[key];	
	}

	public string GetString(string key)
	{
		if (m_database[key].GetType() != typeof(string))
		{
			Debug.LogWarning(key+" should be string but is "+m_database[key].GetType());
		}
		return m_database[key] as string;	
	}
	
	public Vector3 GetVector3(string key)
	{
		if (m_database[key].GetType() != typeof(Vector3))
		{
			Debug.LogWarning(key+" should be vector3 but is "+m_database[key].GetType());
		}
		return (Vector3)m_database[key];	
	}
	
	public float GetFloatWithDefault(string key, float v)
	{
		if (m_database.ContainsKey(key))
		{
			return (float)m_database[key];
		}
		
		//create it and set the default
		
		m_database[key] = v;
		return v;
	}

	public bool GetBoolWithDefault(string key, bool v)
	{
		if (m_database.ContainsKey(key))
		{
			return (bool)m_database[key];
		}
		
		//create it and set the default
		
		m_database[key] = v;
		return v;
	}
	
	public void Set(string key, object v)
	{
		m_database[key] = v;	
	}
	
	public override string ToString()
	{
		string s = "";
		s += "RTDB contains "+m_database.Count+" pairs: ";
		
		foreach (KeyValuePair<string, object> pair in m_database)
		{
		    s += pair.Key + "=" + pair.Value+" ("+pair.Value.GetType()+"), ";
		}	
		
		return s;
	}

	//returns a global static buffer, it's only valid for a microsecond, use it and forget it
	public byte[] Serialize(out int size, short packetType)
	{
		int p =0;
		if (m_database.Count > 255)
		{
			size = 0;
			Debug.Log ("Too big to serialize");
			return null;
		}

	
		p += 4; //skip size, we'll come back
	
		//writing to mem
		RTUtil.SerializeInt16(packetType, RTUtil.g_buffer, ref p);

		RTUtil.SerializeInt16((short)m_database.Count, RTUtil.g_buffer, ref p);

	
		//now let's add each kind of thing we want to serialize
		foreach (KeyValuePair<string, object> pair in m_database)
		{

			RTUtil.SerializeString(pair.Key as string, RTUtil.g_buffer, ref p);

			if (pair.Value.GetType() == typeof(string))
			{
				RTUtil.g_buffer[p] = (byte)TYPE_STRING; p += 1;
				RTUtil.SerializeString(pair.Value as string, RTUtil.g_buffer, ref p);
			} else if (pair.Value.GetType() == typeof(int))
			{
				RTUtil.g_buffer[p] = (byte)TYPE_INT32; p += 1;
				RTUtil.SerializeInt32((int)pair.Value, RTUtil.g_buffer, ref p);
			}  else if (pair.Value.GetType() == typeof(float))
			{
				RTUtil.g_buffer[p] = (byte)TYPE_FLOAT; p += 1;
				RTUtil.SerializeFloat((float)pair.Value, RTUtil.g_buffer, ref p);
			} else if (pair.Value.GetType() == typeof(Vector2))
			{
				RTUtil.g_buffer[p] = (byte)TYPE_VECTOR2; p += 1;
				RTUtil.SerializeVector2((Vector2)pair.Value, RTUtil.g_buffer, ref p);
			} else
			{
				Debug.LogWarning(pair.Key+" can't be serialized, we don't know how to have a "+pair.Value.GetType());
			}
		}	
		//fill in total size now that we know.. writing to index 0 instead of p for this

		int tempIndex = 0;
		RTUtil.SerializeInt32(p, RTUtil.g_buffer,ref tempIndex); //force index 0, and ignore what it returns, we don't care
		size = p;
		return RTUtil.g_buffer;
	}

	public void DeSerialize(byte[] packet)
	{
	
		Clear();

		//read from mem
		int p = 0;

		int totalBytes = 0;
		RTUtil.SerializeInt32(ref totalBytes, packet, ref p);

		short packetType = 0;
		RTUtil.SerializeInt16(ref packetType, packet, ref p);

		short itemCount = 0;
		RTUtil.SerializeInt16(ref itemCount, packet, ref p);


		//grab all items and add as new items

		for (int i=0; i < itemCount; i++)
		{
			string key = "";
			RTUtil.SerializeString(ref key, packet, ref p);
			int type = (int)packet[p]; p += 1; 
		
			switch (type)
			{
			case TYPE_STRING:
			{
				string value = "";
				RTUtil.SerializeString(ref value, packet, ref p);

				//add it
				Set(key, value);
			}
				break;

			case TYPE_INT32:
			{
				int value = 0;
				RTUtil.SerializeInt32(ref value, packet, ref p);
				
				//add it
				Set(key, value);
			}
				break;

			case TYPE_FLOAT:
			{
				float value = 0;
				RTUtil.SerializeFloat(ref value, packet, ref p);
				
				//add it
				Set(key, value);
			}
				break;


			case TYPE_VECTOR2:
			{
				Vector2 vTemp = new Vector2(0,0);
				RTUtil.SerializeVector2(ref vTemp, packet, ref p);
			
				//add it
				Set(key, vTemp);
			}
				break;


			default:

				Debug.LogWarning("Error, don't know what type "+type+" is in DeSerialize");
				return;
			}
		}


		//Debug.Log ("Decrepted size is "+totalBytes+" and itemcount is "+itemCount);
	}
}
